'******************************************************************************
' Copyright (c) 2011, 2012, Dave Hein
' See end of file for terms of use.
'******************************************************************************
CON
  _clkmode = xinput
  _xinfreq = $4E495053 ' SPIN

  CHECKWORD = $dead1eaf
  NEW_LINE = 10
  eeprom_pin = 28
  sdcard_pin = 16

OBJ
  c : "clib_os"
  mem : "cmalloc"
  ser : "cserial"
  eeprom : "pasm_i2c_driver"
  sys : "sysdefs"

DAT
  badflag long 0
  buffer byte 0[100]
  argv long 0[20]
  strbuf byte 0[100]
  
DAT
  redirectin long 0
  redirectout long 0
  commands byte
  byte "dumpm", 0, "peekm", 0, "pokem", 0
  byte "stack", 0, "malloc", 0, "help", 0, "exit", 0
  byte "dump", 0, "peek", 0, "poke", 0, 0
  i2cmsg byte "The I2C driver is not loaded.  Run loadi2c\n", 0

PUB main | i, blocklongs, num, index, outfile, infile, argc, startaddr, endaddr, TopAddr, TopObjList
  'c.printf1(string("long[sys#i2c_cog] = %8.8x\n"), long[sys#i2c_cog])
  REPEAT
    ' Close redirection files if opened
    if redirectin
      c.resetstdin
      c.fclose(redirectin)
      redirectin := 0
    if redirectout
      c.resetstdout
      c.fclose(redirectout)
      redirectout := 0

    c.printf0(STRING("\n>"))
    c.gets(@buffer)
    argc := tokenize(@buffer, @argv)
    argc := Redirection(argc)

    IF (argc == 0)
      NEXT

    num := 0
    index := @commands
    repeat while byte[index]
      if c.strcmp(argv[0], index) == 0
        quit
      index += strsize(index) + 1
      num++

    case num
      0 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.sscanf1(argv[2], STRING("%x"), @num)
        DumpMemory(index, num)
      1 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.printf2(STRING("Memory[%04x] = %02x\n"), index, byte[index])
      2 :
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.sscanf1(argv[2], STRING("%x"), @num)
        byte[index] := num
      3:
        CheckStackSpace(argc, @argv)
      4:
        CheckMallocSpace
      5:
        if (infile := c.fopen(string("/manpages/diaghelp.txt"), string("r")))
          repeat while c.fgets(@buffer, 100, infile)
            c.fputs(@buffer, long[sys#stdout])
          c.fclose(infile)
      6:
        c.exit(0)
      7 :
        ifnot long[sys#i2c_cog]
          c.printf0(@i2cmsg)
          next
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.sscanf1(argv[2], STRING("%x"), @num)
        repeat i from 0 to num - 1
          if ((i & 15) == 0)
            c.printf1(STRING("\n%05x:"), i + index)
          c.printf1(STRING(" %02x"), eeprom.ReadByte(28, $A0, i + index))
        c.printf0(STRING("\n"))
      8 :
        ifnot long[sys#i2c_cog]
          c.printf0(@i2cmsg)
          next
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.printf2(STRING("EEPROM[%04x] = %02x\n"), index, eeprom.ReadByte(28, $A0, index))
      9 :
        ifnot long[sys#i2c_cog]
          c.printf0(@i2cmsg)
          next
        c.sscanf1(argv[1], STRING("%x"), @index)
        c.sscanf1(argv[2], STRING("%x"), @num)
        eeprom.WriteByte(28, $A0, index, num)
        repeat while eeprom.WriteWait(28, $A0, index)
      other:
        i := c.run(argc, @argv, cogid)
        if i < 0
          c.printf1(string("Could not run %s\n"), argv[0])
        else
          index := sys#cogtable + (i * 24)
          repeat while byte[index]

PUB AddToCogTable(cognum, type, name, startaddr, size) | entryptr
  entryptr := sys#cogtable + (cognum * 24)
  byte[entryptr] := type
  bytemove(entryptr+1, name, 19)
  byte[entryptr+19] := 0
  word[entryptr+20] := startaddr
  word[entryptr+22] := size

PUB RemoveFromCogTable(cognum) | entryptr
  entryptr := sys#cogtable + (cognum * 24)
  byte[entryptr] := 0

PUB Redirection(argc) | i, argc1, ptr, char, mode
  if (argc < 2)
    return argc
  argc1 := 1
  repeat i from 1 to argc - 1
    ptr := argv[i]
    char := byte[ptr++]
    if char == "<"
      if (byte[ptr] == 0)
        ptr := argv[++i]
        if (i => argc)
          c.printf0(string("Redirection error\n"))
          argc1 := 0
          quit
      redirectin := c.fopen(ptr, string("r"))
      ifnot redirectin
        c.printf1(string("Could not open %s for input\n"), ptr)
        argc1 := 0
        quit
      else
        c.setstdin(redirectin)
    elseif char == ">"
      if (byte[ptr] == ">")
        ptr++
        mode := string("a")
      else
        mode := string("w")
      if (byte[ptr] == 0)
        ptr := argv[++i]
        if (i => argc)
          c.printf0(string("Redirection error\n"))
          argc1 := 0
          quit
      redirectout := c.fopen(ptr, mode)
      ifnot redirectout
        c.printf1(string("Could not open %s for output\n"), ptr)
        argc1 := 0
        quit
      else
        c.setstdout(redirectout)
    else
      argv[argc1++] := argv[i]
  return argc1

' This routine returns a pointer to the first character not
' matching the passed value
PUB SkipChar(str, value_0)
  REPEAT WHILE (BYTE[str])
    IF (BYTE[str] <> value_0)
      QUIT
    str++
  RETURN str
  
' This routine returns a pointer to the first charcter that
' matches the passed value
PUB FindChar(str, value_0)
  REPEAT WHILE (BYTE[str])
    IF (BYTE[str] == value_0)
      QUIT
    str++
  RETURN str
  
' This routine extracts tokens from the input string that
' are seperated by at least one space character
PUB tokenize(ptr, tokens) | num
  num := 0
  repeat
    ptr := SkipChar(ptr, " ")
    ifnot byte[ptr]
      quit
    num++
    if byte[ptr] == $22
      long[tokens] := ++ptr
      ptr := FindChar(ptr, $22)
    else
      long[tokens] := ptr
      ptr := FindChar(ptr, " ")
    tokens += 4
    IF (BYTE[ptr] == 0)
      QUIT
    BYTE[ptr++] := 0
  RETURN num
  
PUB DumpMemory(address, num) | i, ptr
  ptr := address
  i := 0
  REPEAT WHILE (i < num)
    IF ((i & 15) == 0)
      c.printf1(STRING("\n%05x:"), i + address)
    c.printf1(STRING(" %02x"), BYTE[ptr][i])
    i++
  c.printf0(STRING("\n"))

PUB CheckStackSpace(argc, argv1) | addr, laststackaddr, entryptr, cognum, AppAddr, dbase
  if argc == 2
    c.sscanf1(long[argv1][1], string("%d"), @cognum)
    
  if argc <> 2 or cognum < 0 or cognum > 7
    c.printf0(string("usage: stack cogid\n"))
    return
    
  entryptr := sys#cogtable + 24 * cognum
  if (byte[entryptr] & 15) <> sys#proc_type_spin
    c.printf1(string("Cog %d is not running"), cognum)
    if byte[entryptr]
      c.printf0(string(" a Spin program"))
    c.printf0(string("\n"))
    return

  AppAddr := word[entryptr + 20]

  dbase := word[AppAddr][5]
  
  laststackaddr := dbase + 800 - 4
  repeat addr from laststackaddr to 0 step 4
    if long[addr] <> sys#CHECKWORD
      quit
    result++
  c.printf1(string("The last %d longs have not been used on the stack\n"), result)
  
PUB CheckMallocSpace | ptr, numalloc, numfree
  numalloc := 0
  numfree := 0
  ptr := long[sys#malloclist]
  c.printf0(string("MallocList\n"))
  repeat while(ptr)
    c.printf2(string("addr = %04x size = %d\n"), ptr, word[ptr][1])
    numalloc += word[ptr][1]
    ptr := word[ptr]
  ptr := long[sys#memfreelist]
  c.printf0(string("FreeList\n"))
  repeat while(ptr)
    c.printf2(string("addr = %04x size = %d\n"), ptr, word[ptr][1])
    numfree += word[ptr][1]
    ptr := word[ptr]
  c.printf2(string("%d bytes allocated, %d bytes free\n"), numalloc, numfree)

{{
+-----------------------------------------------------------------------------+
|                       TERMS OF USE: MIT License                             |
+-----------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy |
|of this software and associated documentation files (the "Software"), to deal|
|in the Software without restriction, including without limitation the rights |
|to use, copy, modify, merge, publish, distribute, sublicense, and/or sell    |
|copies of the Software, and to permit persons to whom the Software is        |
|furnished to do so, subject to the following conditions:                     |
|                                                                             |
|The above copyright notice and this permission notice shall be included in   |
|all copies or substantial portions of the Software.                          |
|                                                                             |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   |
|IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     |
|FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE  |
|AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER       |
|LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,|
|OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE|
|SOFTWARE.                                                                    |
+-----------------------------------------------------------------------------+
}}
